# -*- coding: utf-8 -*-
'''
Copyright 2012 Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
from argparse import ArgumentParser, Action
from jinja2 import Template

import os
import re
import unicodedata as u

try:
    from PIL import Image
    from PIL import ImageColor
except ImportError:
    import Image
    import ImageColor

DEBUG = False
DEBUG = True

get_namefile = lambda n: '.'.join(n.split('.')[:-1])

def get_long_description():
    try:
        text = open('README.rst', 'r').read()
    except:
        text = None
    finally:
        return text

def get_infos():
    return {
        'name': 'spiritify',
        'description': 'The spiritify is a tool for transform directory of images in the stylesheet and image file.',
        'long_description': get_long_description(),
        'version': version(),
        'home': 'https://bitbucket.org/rodrigopmatias/spiritify',
        'url': 'https://bitbucket.org/rodrigopmatias/spiritify/downloads',
        'author': 'Rodrigo Pinheiro Matias',
        'author_email': 'rodrigopmatias@gmail.com',
        'maintainer': 'Rodrigo Pinheiro Matias',
        'maintainer_email': 'rodrigopmatias@gmail.com',
        'py_modules': ['res'],
        'scripts': ['spiritify.py'],
        'install_requires': ['PIL', 'jinja2'],
        'packages': ['templates',],
        'package_data': {'templates': ['templates/*.html']}
    }

def version():
    return '0.1.5'

def factory_args_parse():
    parser = ArgumentParser(
        description = u'HTML Generator of thumb directory.'
    )

    parser.add_argument(
        '-v',
        '--version',
        help =  u'Print the version of generator.',
        action = 'store_true'
    )

    parser.add_argument(
        '-p',
        '--path',
        help = u'Path of base images to show in thum',
        dest = 'path'
    )

    parser.add_argument(
        '-o',
        '--output',
        help = u'Path of file to out processed html with thumbs.',
        dest = 'outfile'
    )

    parser.add_argument(
        '-t',
        '--template',
        help = u'HTML Template.',
        dest = 'template',
        default = 'default.html'
    )

    parser.add_argument(
        '-s',
        '--spirite',
        help = u'Spirite name for css and HTML.',
        dest = 'spirite_name',
        default = None
    )

    return parser

def get_do_params(nm):
    dct = nm.__dict__
    del dct['version']

    return dct

def slugify(text):
    text = u.normalize('NFKD', unicode(text)).lower()
    return re.sub('(-|\.|_|\s|\(|\)|\[|\]|\{|\}|\'|\")+', '-', text)

def get_images_from_path(path, append = None):
    images = {}

    for filename in os.listdir(path):
        if re.match('^.+\.(png|jpg|gif|tiff|bmp)$', filename) is not None:
            try:
                im = Image.open(os.path.join(path, filename))
            except:
                print 'ERR: %s' % os.path.join(path, filename)
            else:
                slot = im.size
                layers = images.get(slot, [])
                to_slug = get_namefile(filename) if append is None else ' '.join([get_namefile(filename), append])
                layers.append((im, slugify(to_slug)))
                images.update({slot: layers})
        elif filename not in ('.', '..') and os.path.isdir(os.path.join(path, filename)):
            images.update(get_images_from_path(
                os.path.join(path, filename),
                filename if append is None else '-'.join([append, filename])
            ))

    return images

def get_image_out_size(images):
    out_width = 0
    out_height = 0

    for key in images.keys():
        width, height = key
        out_width += width
        out_height = height * len(images.get(key)) if height * len(images.get(key)) > out_height else out_height

    return out_width, out_height


def do(path, template, outfile, minify = False, spirite_name = None):
    outdir = os.path.dirname(outfile.name)

    if spirite_name is None:
        spirite_name = '-'.join(['spirite', slugify(os.path.basename(get_namefile(outfile.name)))])

    spirite_css_file = os.path.join(outdir, '.'.join([spirite_name, 'css']))
    spirite_image_file = os.path.join(outdir, '.'.join([spirite_name, 'png']))
    css_file = open(spirite_css_file, 'wt')

    images = get_images_from_path(path)
    out_width, out_height = get_image_out_size(images)

    transparent = (255, 255, 255, 0)
    out_im = Image.new('RGBA', (out_width, out_height), color = transparent)


    css_lines = []
    css_lines.append('''.icon-%(slug)s { background: url('%(image)s') no-repeat 0 0 } ''' % {
        'slug': spirite_name,
        'image': os.path.basename(spirite_image_file)
    })

    left = 0
    top = 0
    for slot, items in images.items():
        width, height = slot
        for item in items:
            im, slug = item
            out_im.paste(im, (left, top))
            css_lines.append('''.icon-%(slug)s { background-position: %(left)dpx %(top)spx !important } ''' % {
                'slug': slug,
                'top': top * (-1),
                'left': left * (-1)
            })
            top += height
        left += width
        top = 0

    tpl = Template(template.read())
    icon = lambda x: 'icon-%s' % x
    outfile.write(tpl.render(
        cssfile =  os.path.basename(css_file.name),
        slots = [
        {
            'width': slot[0],
            'height': slot[1],
            'images': [
                ' '.join([icon(spirite_name), icon(slug)]) 
                for im, slug in items
            ]
        } for slot, items in images.items()
    ]))

    out_im.save(spirite_image_file, 'PNG')
    css_file.write('\n'.join(css_lines) if minify is False else ' '.join(css_lines))
    css_file.close()
